      subroutine DYNadv_sal
 
C +------------------------------------------------------------------------+
C | MAR DYNAMICS   SLOW                               Tue 10-Jan-2012  MAR |
C |   SubRoutine DYNadv_sal includes the Horizontal Advection Contribution |
C |                                  of Saltating Snow                     |
C |                                                                        |
C +------------------------------------------------------------------------+
C |                                                                        |
C |   INPUT   (via common block)                                           |
C |   ^^^^^    iterun: Run   Iteration                        Counter      |
C |            micphy: Cloud Microphysics                     Switch       |
C |            openLB: Open  Lateral Boundary Condition (LBC) Switch       |
C |                                                                        |
C |   INPUT  / uairDY,vairDY, qsHY, uss_HY                                 |
C |   OUTPUT : snobSL  : snow eroded        thickness            [m w.e.]  |
C |   ^^^^^^^^ snohSL  : snow precipitation thickness            [m w.e.]  |
C |                                                                        |
C |   METHOD : The following Contributions are taken into account:         |
C |   ^^^^^^^^     dq/dt:=-(ud(qp*)/dx -vd(qp*)/dy)/p*                     |
C |                                                                        |
C |   REFER. : Alpert, thesis, 1980                                        |
C |   ^^^^^^^^ Pielke, Mesoscale Meteorological Modeling, 297--307, 1984   |
C |            Seibert and Morariu, JAM, p.118, 1991                       |
C |                                                                        |
C +------------------------------------------------------------------------+
 
 
      IMPLICIT NONE
 
 
C +--Global Variables
C +  ================
 
      include 'MARCTR.inc'
      include 'MARphy.inc'
 
      include 'MARdim.inc'
      include 'MARgrd.inc'
 
      include 'MAR_DY.inc'
      include 'MAR_HY.inc'
      include 'MAR_BS.inc'
 
      include 'MAR_SL.inc'
 
      include 'MAR_WK.inc'
 
 
C +--Local  Variables
C +  ================
 
      real                   facFLX
      common  /DYNadv_sal_r/ facFLX
 
 
      IF (iterun.LE.1)                                              THEN
!         IF (zsigma(mz).LT.1.5)
!    .    STOP ' in DYNadv_sal: z_sbl inacceptable'
!         facFLX=0.1+0.8*exp(-(zsigma(mz)-0.5)*0.15)
          facFLX=2.0+6.0*exp(-(zsigma(mz)-0.4)*1.50) - 1.
      END IF
 
 
 
!  +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 
 
!  Additional Snow Particles Horizontal Flux
!  =========================================
 
         k = mz
!        k = mz (i.e., Consider SBL only)
 
      DO j=1,my
      DO i=1,mx
        WKxy0(i,j) = pstDY(i,j)
     . *facFLX     *  qsHY(i,j,k)*min(unun,sign(unun,zero-uss_HY(i,j)))
      END DO
      END DO
 
 
      DO j=1,my
      DO i=1,mx
        WKxy1(i,j) = WKxy0(i,j)*uairDY(i,j,k)
        WKxy2(i,j) = WKxy0(i,j)*vairDY(i,j,k)
      END DO
      END DO
 
 
!  Flux Convergence
!  ================
 
      DO j=jp11,my1
      DO i=ip11,mx1
        qsHY(i,j,k) = qsHY(i,j,k)
     .        +(dtx*(WKxy1(im1(i),    j )-WKxy1(ip1(i),    j ))
     .         +dty*(WKxy2(    i ,jm1(j))-WKxy2(    i ,jp1(j))))
     .        /(2.0* pstDY(    i,     j))
      END DO
      END DO
 
 
      DO j=1,my
      DO i=1,mx
        WKxy0(i,j) = 0.
        WKxy1(i,j) = 0.
        WKxy2(i,j) = 0.
      END DO
      END DO
 
 
!  +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 
 
      return
      end
