 
 
      subroutine MAR_2_OASIS
 
! +--------------------------------------------------------------------------+
! |                                                                1-12-2019 |
! |   subroutine MAR_2_OASIS transfers MAR variables to OASIS                |
! |                                                                          |
! |                                                                          |
! |      A chaque pas de tps, apres fromcpl, intocpl est appele ici;         |
! |      si il_time_secs correspond au pas de temps precedant le couplage,   |
! |      alors son action est effective        (via prism_put),              |
! |         et MAR envoie ses champs            via givfld (oasis)           |
! |                       a NEMO au pas de temps du couplage.                |
! |      L'appel a chaque pas de temps sert au moins au calcul de moyennes   |
! |                                            (cf AVERAGE dans namcouple)   |
! |                                                                          |
! |      cf mar_module.f90                                                   |
! |      cf OASIS-3 user guide (december 2004)                               |
! +--------------------------------------------------------------------------+
 
c #AO USE mod_oasis
c #AO USE mar_module      !TANGO modules

 
      IMPLICIT NONE
 
! +--MAR    Variables
! +  ----------------
 
c #AO include 'MARCTR.inc'
c #AO include 'MARphy.inc'
 
c #AO include 'MARdim.inc'
c #AO include 'MARgrd.inc'
c #AO include 'MAR_GE.inc'
 
c #AO include 'MAR_DY.inc'
 
c #AO include 'MAR_HY.inc'
c #AO include 'MAR_RA.inc'
 
c #AO include 'MAR_SL.inc'
c #AO include 'MAR_AO.inc'
 
c #AO include 'MAR_WK.inc'
 
c #AO integer n
      
     
 
! +--cpl-1 Let's extract or calculate the usefull fields
! +  ===================================================
 
c #AO   IF (iterun.eq.0)                                            THEN
c #AO       write(6,*) 'Ini summed precip for OASIS'
c #AO	    write(6,*) 'MAR first time step :  iterun = ',iterun
c #AO       do i=1,mx; do j=1,my
c #AO         lprecipAO_t2(i,j)=0
c #AO         sprecipAO_t2(i,j)=0
c #AO	    enddo; enddo
c #AO   END IF 

	!cleaning	
c #AO	do i=1,mx; do j=1,my
c #AO   evapAO(i,j)=0.
c #AO	ievpAO(i,j)=0.
c #AO	lprecipAO(i,j)=0.
c #AO	sprecipAO(i,j)=0.	
c #AO	enddo; enddo
       
        !rotation of wind on a regular grid
c #AO  call wind_rot(UairDY(:,:,mz),VairDY(:,:,mz),uuao,vvao)

 
c #AO   DO i=2,mx-1
c #AO   DO j=2,my-1
c #AO       lprecipAO_t1(i,j)=lprecipAO_t2(i,j)                                          !total liq. precip since iterun=0 at dt-1
c #AO       lprecipAO_t2(i,j)=rainHY(i,j)                                                !total liq. precip since iterun=0 at dt
c #AO       lprecipAO(i,j)   =(lprecipAO_t2(i,j)-lprecipAO_t1(i,j))                        !liquid precip between dt-1 and dt (mm.w.e)	    
	    
c #AO       sprecipAO_t1(i,j)=sprecipAO_t2(i,j)                                          !total sol. precip since iterun=0 dt-1
c #AO       sprecipAO_t2(i,j)=snowHY(i,j)+crysHY(i,j)                                    !total sol. precip since iterun=0 (at itexpe)
c #AO       sprecipAO(i,j)   =(sprecipAO_t2(i,j)-sprecipAO_t1(i,j))                        !solid  precip between dt-1 and dt (mm.w.e)
 
            !Conversion m.w.e to kg.m-2.s-1
c #AO       lprecipAO(i,j)=lprecipAO(i,j)*1000./dt
c #AO	    sprecipAO(i,j)=sprecipAO(i,j)*1000./dt
 
! for blowing snow ?
!            IF (sprecipAO(i,j).ge.0.)                               THEN
!                 upsnowAO(i,j) =  0.
!            ELSE
!                 upsnowAO(i,j) = -sprecipAO(i,j)
!                sprecipAO(i,j) =  0.
!            END IF 
 
 
c #AO        evapAO(i,j) = -   SLuqsl(i,j,1) *rolvDY(i,j,mz) *1.e3                       !evaporation over ocean (negative, kg.m-2.s-1)
c #AO        ievpAO(i,j) = -   SLuqsl(i,j,2) *rolvDY(i,j,mz) *1.e3                       !sublimation over sea ice (negative, kg.m-2.s-1)
c #AO        evapAO(i,j) = evapAO(i,j) - ievpAO(i,j)                                     !Evap total (ocean + sublimation)
 
             !Utilise maintenant vent rot au lieu de uairDY, vairDY 


 
c #AO       DO n=1,2
c #AO	     if (n .eq. 1) then ! ocean
c #AO         cpa        = cp*(1+qvswDY(i,j,mz)*cpvir)
		                                
c #AO         radsolAO(i,j,n) = RAdsol(i,j)  * ( 1.0 - albAO(i,j,n) )                           ! downward solar                        (W.m-2)
c #AO         radirAO(i,j,n) = RAd_ir(i,j)
c #AO.                        - EmiWatao*stefan *     tsrfsl(i,j,n)**4                         ! downward IR                      (net, W.m-2)
c #AO         hsenAO(i,j,n) = SLutsl(i,j,n)*cpa*rolvDY(i,j,mz)*1.e3                             ! downward sensible                     (W.m-2)
c #AO         hlatAO(i,j,n) = SLuqsl(i,j,n)*rolvDY(i,j,mz)*1.e3*Lv_H2O                          ! downward latent                       (W.m-2)
c #AO         radtotAO(i,j,n) = radirAO(i,j,n)
c #AO.                        +  hsenAO(i,j,n)                                                   ! downward non solar                    (W.m-2) 
c #AO.                        !+ hlatAO(i,j,n)     double counted in NEMO?                            

!UoceAO en dehors de domaine NEMO mais dans MAR= valeur plus proche voisin dans NEMO lors du couplage. Evite les 0 sur les bords?

c #AO         normdUAO      = sqrt((uuao(i,j) - UoceAO(i,j))**2
c #AO.                               +(vvao(i,j) - VoceAO(i,j))**2)

c #AO          TauxtAO(i,j,n) =cdmSL(i,j,n)**2 *normdUAO*rolvDY(i,j,mz)
c #AO.                       *(uuao(i,j)-UoceAO(i,j)) *1.e3                                ! wind stress on T grid                 (N.m-2)
c #AO          TauytAO(i,j,n) =cdmSL(i,j,n)**2 *normdUAO*rolvDY(i,j,mz)
c #AO.                       *(vvao(i,j)-VoceAO(i,j)) *1.e3


c #AO        endif    
c #AO	     if (n .eq. 2) then ! sea ice
c #AO         cpa        = cp*(1+ qvsiDY(i,j,mz)*cpvir)                                         ! moist air heat capacity for ice
 
c #AO         radsolAO(i,j,n) = RAdsol(i,j)  * ( 1.0 - albAO(i,j,n) )                           ! downward solar                        (W.m-2)
c #AO         radirAO(i,j,n) = RAd_ir(i,j)
c #AO.                        - Emisnoao*stefan *   tsrfsl(i,j,n)**4                            ! downward IR                      (net, W.m-2)
c #AO         hsenAO(i,j,n) = SLutsl(i,j,n)*cpa*rolvDY(i,j,mz)*1.e3                             ! downward sensible                     (W.m-2)
c #AO         hlatAO(i,j,n) = SLuqsl(i,j,n) *rolvDY(i,j,mz)*1.e3*Ls_H2O                         ! downward latent                       (W.m-2)
c #AO         radtotAO(i,j,n) = radirAO(i,j,n)
c #AO.                        +  hsenAO(i,j,n)                                                  ! downward non solar                    (W.m-2)
c #AO.                        ! + hlatAO(i,j,n)                                  

! Derivee des flux non solaire selon la temperature pour eviter les oscillations
c #AO          DFlAO(i,j,n) = -rolvDY(i,j,mz)*SLuusl(i,j,n)
c #AO.                        *  cdhSL(i,j,n)*R_Rv*1.e3
c #AO.                        * (Ls_H2O )**2
c #AO.                        * (qvsiDY(i,j,mzz))
c #AO.                        / (       RDryAi *       tsrfsl(i,j,n)**2)                        ! latent flux derivative/T Clausius-Clapeyron for Dqsat/DT)
c #AO          DFsAO(i,j,n) = -rolvDY(i,j,mz)*cpa*1.e+3
c #AO.                        *   cdhSL(i,j,n) *SLuusl(i,j,n)                                   ! sensible flux derivative/T
c #AO          DFiAO(i,j,n) = -4*epsAO*stefan*tsrfsl(i,j,n)**3                                  ! IR flux derivative/T
c #AO          DFtotAO(i,j,n) =  DFlAO(i,j,n)+DFsAO(i,j,n)+DFiAO(i,j,n)                         !  total         derivative/T
               
!UiceAO ViceAO en dehors de domaine NEMO mais dans MAR= valeur plus proche voisin dans NEMO lors du couplage. Evite les 0 sur les bords?
c #AO          normdUAO      = sqrt((uuao(i,j) - UiceAO(i,j))**2
c #AO.                               +(vvao(i,j) - ViceAO(i,j))**2)  

!uairday and vairday has been changed by rorated wind
c #AO          TauxtAO(i,j,n) =cdmSL(i,j,n)**2 *normdUAO*rolvDY(i,j,mz)
c #AO.                       *(uuao(i,j)-UiceAO(i,j)) *1.e3                                              !wind stress on T grid                 (N.m-2)
c #AO          TauytAO(i,j,n) =cdmSL(i,j,n)**2 *normdUAO*rolvDY(i,j,mz)
c #AO.                       *(vvao(i,j)-ViceAO(i,j)) *1.e3


c #AO       endif
 

c #AO       ENDDO
            


c #AO   ENDDO
c #AO   ENDDO
 
 
c #AO   DO n=1,2
c #AO     DO i=1,mx-1
c #AO     DO j=1,my-1
c #AO       TauxuAO(i,j,n)= 0.5*(TauxtAO(i,j,n) + TauxtAO(i+1,j,n)) ! U grid
c #AO       TauyuAO(i,j,n)= 0.5*(TauytAO(i,j,n) + TauytAO(i+1,j,n)) ! U grid
c #AO       TauxvAO(i,j,n)= 0.5*(TauxtAO(i,j,n) + TauxtAO(i,j+1,n)) ! V grid
c #AO       TauyvAO(i,j,n)= 0.5*(TauytAO(i,j,n) + TauytAO(i,j+1,n)) ! V grid
c #AO     ENDDO
c #AO     ENDDO
c #AO     DO i=1,mx !bord des grilles
c #AO       TauxvAO(i,my,n)=     TauxtAO(i,my,n)
c #AO       TauyvAO(i,my,n)=     TauxtAO(i,my,n)
c #AO     ENDDO
c #AO     DO j=1,my
c #AO       TauxuAO(mx,j,n)=     TauxtAO(mx,j,n)
c #AO       TauyuAO(mx,j,n)=     TauxtAO(mx,j,n)
c #AO     ENDDO
c #AO   ENDDO

c #AO   do i=1,mx; do j=1,my; do n=1,2
c #AO     if (maskSL(i,j) .ne. 1) then
c #AO      TauxuAO(i,j,n)=0.
c #AO      TauyuAO(i,j,n)=0.
c #AO      TauxvAO(i,j,n)=0.
c #AO      TauyvAO(i,j,n)=0.
c #AO	   DFtotAO(i,j,n)=0.
c #AO     endif
c #AO   enddo; enddo; enddo
 
 
! +--cpl-2 Now let's give the fields to oasis
! +  ========================================
 
c #AO     il_time_secs = iterun*idt
! +...                   temps en sec depuis le debut du run, au pas precedant
cc #AO     write(*,*) '     CALL intocpl, itexpe =', itexpe !for debug
 


c #AO	  CALL intocpl(il_time_secs,   radsolAO(:,:,2),radsolAO(:,:,1),
c #AO.  	       radtotAO(:,:,2),radtotAO(:,:,1), DFtotAO(:,:,2),
c #AO.  		 evapAO       ,lprecipAO,     sprecipAO       ,
c #AO.  	       ievpAO,
c #AO.  		TauxuAO(:,:,1), TauxuAO(:,:,2),
c #AO.  		TauyuAO(:,:,1), TauyuAO(:,:,2),
c #AO.  		TauxvAO(:,:,1), TauxvAO(:,:,2),
c #AO.  		TauyvAO(:,:,1), TauyvAO(:,:,2))
! +            *******
 
 
      return
      end


