 
 
      subroutine OASIS_2_MAR
 
C +--------------------------------------------------------------------------+
C |                                                                20-20-2020|
C |   subroutine OASIS_2_MAR transfers OASIS variables to MAR                |
C |                                                                          |
C |                                                                          |
C |      A chaque pas de temps, fromcpl est appele ;                         |
C |      si il_time_secs est egal a un multiple de la periode de couplage    |
C |      alors son action est effective        (via oasis_get),              |
C |         et MAR recupere les champs de NEMO  via getfld (oasis)           |
C |                                                                          |
C |     C. Kittel + P-V. Huot + N. Jourdain                                  |
C |      cf mar_module.f90                                                   |
C |      cf OASIS-3 user guide (december 2004)                               |
C +--------------------------------------------------------------------------+
 
c #AO USE mod_oasis !
c #AO USE mar_module    

 
      IMPLICIT NONE
 
C +--MAR    Variables
C +  ----------------

c #AO include  "MARCTR.inc"
c #AO include  "MARphy.inc"

c #AO include  "MARdim.inc"

c #AO include  "MARgrd.inc"
c #AO include  "MAR_GE.inc"

c #AO include  "MAR_RA.inc"

c #AO include  "MAR_LB.inc"
c #AO include  "MAR_DY.inc"
c #AO include  "MAR_HY.inc"
c #BW include  "MAR_TE.inc"
c #AO include  "MAR_TU.inc"

c #TC include  "MAR_TC.inc"

c #AO include  "MAR_SV.inc"
c #AO include  "MARdSV.inc"
c #AO include  "MAR0SV.inc"

c #AO include  "MAR_SL.inc"
c #AO include  "MAR_AO.inc"
c #AO include  "MAR_TV.inc"
c #AO include  "MAR_BS.inc"
c #AO include  "MARsSN.inc"
c #AO include  "MAR_IB.inc"

c #AO include  "MARsIB.inc"
c #PO include  "MAR_PO.inc"

c #AO include  "MAR_WK.inc"

c #AO integer n
c #AO  integer isl
 
! AO_CK 20/02/2020

C +--cpl : GET FIELDS FROM OASIS
C +  ===========================
cc #AO       write(6, *)  "get fields?", tocken_AO 
c #AO IF (tocken_AO.eq.1) THEN
c #AO   tocken_AO=0      !fromcpl a deja ete appele pour ce pas de temps
c #AO ELSE
c #AO      il_time_secs = iterun*idt
C +...                    temps du run en sec. au pas de temps precedant
cc #AO     write(*,*) '     CALL fromcpl, itexpe =', itexpe
 
C +             *******
c #AO      CALL fromcpl(il_time_secs, srftAO(:,:,1),aoss,
c #AO.     sicsAO,aogla,albAO(:,:,2),aoalb, srftAO(:,:,2),aotic,
c #AO.     hicAO,aohic, hsnoAO, aohsn,UoceAO,ao_uo, VoceAO,ao_vo, 
c #AO.                  UiceAO,ao_ui, ViceAO,ao_vi)
C +             *******
         
          !cpl----those oceanic fields from oasis are used along the code--
 
C +--open water albedo (same parametrisation as in NEMO,
c +  -----------------  but  with a real zenith angle)
 
                       ! see Briegleb & Ramanathan, JAM, 1982
cc #AO     print *, aoss, il_time_secs,iterun !for debug
c #AO    do i=1,mx; do j=1,my
	  
	  !Albedo from the ocean as computed in NEMO 
	  !czenGE=cos(zenith angle),cld_SL=cloud cover
c #AO     albAO(i,j,1)=( 0.05/(1.1*czenGE(i,j)**1.4+0.15) )*cld_SL(i,j)
c #AO.                +  0.06                        *  (1.-cld_SL(i,j))
     
                                  
c #AO     if (aoss .gt. 0 .and. weightao_sst(i,j) .ne. 1) then !you should avoid coupling sst and sic on different time steps!
                                                               !even if it's possible in the code with different aoss and aogla
	   
	   
c #AO	    srftAO(i,j,1) = min(max(srftAO(i,j,1)/(1-sicsAO(i,j)) !avoid interpolation error in oasis
c #AO.     ,271.01),300.15)                                       !WARNING WARNING WARNING                    

	  
c #AO      sst_LB(i,j)= (1.-weightao_sst(i,j))*srftAO(i,j,1)
c #AO.             +(weightAO_sst(i,j)*sst_LB(i,j))
c #AO	  endif
	  
	  
c #AO	  if (aotic .gt. 0 .and. weightao_st(i,j) .ne. 1) then !+update of new snow layers in phy_sisvat_mp.f
c #AO	    srftAO(i,j,2) = min(max(srftAO(i,j,2)/sicsAO(i,j)    !avoid interpolation error in oasis (non interecative mask) !WARNING WARNING WARNING
c #AO.                         ,250.001),273.15)

c #AO      if (sicsAO(i,j) .gt. 0) then
c #AO       DO isl =         1,nsno 
c #AO	     if (maskSL(i,j) .eq. 1 .and. nssSNo(i,j,2) .ge. 1) then 
	     !if there  was snow/sea ice at dt-1 on sea(maskSL=1)               
c #AO         tisSNo(i,j,2,isl)   = (1.-weightao_st(i,j))*srftAO(i,j,2) 
c #AO.             +weightAO_st(i,j)*tisSNo(i,j,2,isl)
c #AO	     endif        
c #AO       END DO
c #AO	   endif

c #AO	  endif

c #AO     if (aogla .gt. 0 .and. weightAO_sic(i,j) .ne. 1) then
c #AO      sicsIB(i,j) =  (1.-weightao_sic(i,j))* sicsAO(i,j)
c #AO.                   +(weightAO_sic(i,j)*sicsIB(i,j)) 
c #AO	  endif

c #AO	  if (aoalb .gt. 0) then
c #AO	  albAO(i,j,2) = max(min(albAO(i,j,2)/sicsAO(i,j),1.0),0.0)
	  !weighting directly in snoptp.f (phy_sisvat => (sisvat) => snoptp)
c #AO	  endif
	  
c #AO     enddo; enddo
	  

	  
        !hicAO et hsnoAO => only in full NEMO domain (update in physisvat_mp)


c #AO END IF
 
      return
      end

